package system

import (
	"fmt"
	"strings"

	gnmitargetygot "code.fbi.h-da.de/danet/gnmi-target/examples/example01/model"
	"code.fbi.h-da.de/danet/gnmi-target/examples/example01/osclient"
	"code.fbi.h-da.de/danet/gnmi-target/handler"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

// SystemHandler is the implementation of a gnmitarget.PathHandler.
type SystemHandler struct {
	handler.DefaultPathHandler
	osClient osclient.Osclient
}

func NewSystemHandler() *SystemHandler {
	return &SystemHandler{
		DefaultPathHandler: handler.DefaultPathHandler{
			Name: "openconfig-system-handler",
			Paths: map[string]struct{}{
				"/system": {},
			},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *SystemHandler) Init(config *handler.Config, publishToSubsFunc func([]*gnmi.Notification) error) error {
	yh.Config = config
	yh.PublishToSubs = publishToSubsFunc

	_, err := yh.updateOrCreateSystem(yh.osClient)
	if err != nil {
		return err
	}
	return nil
}

func (yh *SystemHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.Name)
	return nil
}

func (yh *SystemHandler) updateOrCreateSystem(os osclient.Osclient) ([]*gnmi.Notification, error) {
	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return nil, err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return nil, fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	confSystem := newConfig.GetOrCreateSystem()

	if config := confSystem.GetOrCreateConfig(); config != nil {
		domain, err := os.GetDomainName()
		if err != nil {
			return nil, err
		}
		config.DomainName = ygot.String(domain)
	}

	if clock := confSystem.GetOrCreateClock(); clock != nil {
		if config := clock.GetOrCreateConfig(); config != nil {
			config.TimezoneName = ygot.String(strings.TrimSuffix(os.GetTimeZoneName(), "\n"))
		}
	}

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return nil, err
	}

	notifications, err := ygot.DiffWithAtomic(yh.Config.Data, newConfig)
	if err != nil {
		return nil, err
	}

	yh.Config.Data = newConfig

	return notifications, nil
}
