package additions

import (
	"fmt"
	"net"

	gnmitargetygot "code.fbi.h-da.de/danet/gnmi-target/examples/example01/model"
	log "github.com/sirupsen/logrus"

	"github.com/vishvananda/netlink"
	"github.com/vishvananda/netlink/nl"
)

func (oc *networkInstances) SetStaticRoute(staticRoute *StaticRoute) error {
	for _, hop := range staticRoute.Hops {
		log.Info(*hop.InterfaceRef)
		link, err := netlink.LinkByName(*hop.InterfaceRef)
		if err != nil {
			return err
		}

		ipnet, err := cidrToIPNet(*staticRoute.Prefix)
		if err != nil {
			return err
		}
		nextHop, ok := hop.NextHop.(*gnmitargetygot.OpenconfigNetworkInstance_NetworkInstances_NetworkInstance_Protocols_Protocol_StaticRoutes_Static_NextHops_NextHop_Config_NextHop_Union_String)
		if !ok {
			return fmt.Errorf("only string union is supported for nextHop")
		}

		nextHopParsed := net.ParseIP(nextHop.String)

		if nextHop == nil {
			return fmt.Errorf("failed parsing ip")
		}

		route := &netlink.Route{
			LinkIndex: link.Attrs().Index,
			Dst:       ipnet,
			Gw:        nextHopParsed,
		}

		log.Debug("Created netlink route: ", route.String())

		if err := netlink.RouteReplace(route); err != nil {
			log.Debug("Failed route replace: ", err)
			return err
		}
	}

	return nil
}

func (oc *networkInstances) GetStaticRoutes(linkName string) ([]*StaticRoute, error) {
	// Create Interfaces
	h, err := netlink.NewHandle()
	if err != nil {
		log.WithFields(log.Fields{}).Error(err)
	}

	link, err := h.LinkByName(linkName)
	if err != nil {
		return nil, err
	}

	localRoutesV4, err := h.RouteList(link, nl.FAMILY_V4)
	if err != nil {
		log.WithFields(log.Fields{}).Error(err)
	}

	localRoutesV6, err := h.RouteList(link, nl.FAMILY_V6)
	if err != nil {
		log.WithFields(log.Fields{}).Error(err)
	}

	staticRoutes := make([]*StaticRoute, 0)
	for _, localRoute := range localRoutesV4 {
		if localRoute.Gw != nil && localRoute.Src == nil {
			staticRoutes = append(staticRoutes, staticRouteBasedOnFamily(link, localRoute, nl.FAMILY_V4))
		}
	}

	for _, localRoute := range localRoutesV6 {
		if localRoute.Gw != nil && localRoute.Src == nil {
			staticRoutes = append(staticRoutes, staticRouteBasedOnFamily(link, localRoute, nl.FAMILY_V6))
		}
	}

	return staticRoutes, nil
}

func staticRouteBasedOnFamily(link netlink.Link, route netlink.Route, family int) *StaticRoute {
	prefix := ipNetIsNilConverter(route.Dst, family)
	nextHop := &gnmitargetygot.OpenconfigNetworkInstance_NetworkInstances_NetworkInstance_Protocols_Protocol_StaticRoutes_Static_NextHops_NextHop_Config_NextHop_Union_String{String: route.Gw.String()}
	index := fmt.Sprintf("AUTO_%s", nextHop.String)
	return &StaticRoute{
		Prefix: &prefix,
		Hops: []Hop{
			Hop{
				InterfaceRef: &link.Attrs().Name,
				Index:        &index,
				NextHop:      nextHop,
			},
		},
	}
}

func ipNetIsNilConverter(ipNet *net.IPNet, family int) string {
	if ipNet != nil {
		return ipNet.String()
	}

	switch family {
	case nl.FAMILY_V4:
		return "0.0.0.0/0"
	case nl.FAMILY_V6:
		return "::/0"
	default:
		return ipNet.String()
	}
}

func cidrToIPNet(s string) (*net.IPNet, error) {
	ip, ipnet, err := net.ParseCIDR(s)
	if err != nil {
		return nil, err
	}
	ipnet.IP = ip

	return ipnet, nil
}
